#![feature(abi_thiscall)]

use std::ffi::{c_void, CStr};

pub use platform::current::BasePlugin;

mod ifilesystem;
mod inetworkstringtable;
mod platform;

pub use inetworkstringtable::{
    INetworkStringTable,
    INetworkStringTableContainer
};

#[repr(i32)]
#[derive(Debug, Clone, Copy)]
pub enum PluginResult {
    Continue,
    Override,
    Stop,
}

#[repr(i32)]
#[derive(Debug, Clone, Copy)]
pub enum QueryStatus {
    ValueIntact,
    CvarNotFound,
    NotACvar,
    CvarProtected,
}

pub type CreateInterfaceFn = unsafe extern "C" fn(*const i8, *mut i32) -> *mut c_void;

#[allow(unused_variables)]
pub trait ServerPlugin {
    fn load(
        &mut self,
        game_factory: CreateInterfaceFn,
        server_factory: CreateInterfaceFn,
    ) -> Result<(), Box<dyn std::error::Error>> {
        Ok(())
    }
    fn unload(&mut self) {}
    fn pause(&mut self) {}
    fn unpause(&mut self) {}
    fn get_plugin_description(&mut self) -> &CStr;
    fn level_init(&mut self, level_name: &CStr) {}
    fn server_activate(&mut self, edict_list: &mut [*mut c_void], max_clients: i32) {}
    fn game_frame(&mut self, simulating: bool) {}
    fn level_shutdown(&mut self) {}
    fn client_active(&mut self, edict: *mut c_void) {}
    fn client_disconnect(&mut self, edict: *mut c_void) {}
    fn client_put_in_server(&mut self, edict: *mut c_void, player_name: &CStr) {}
    fn set_command_client(&mut self, client_index: i32) {}
    fn client_settings_changed(&mut self, edict: *mut c_void) {}
    fn client_connect(
        &mut self,
        edict: *mut c_void,
        player_name: &CStr,
        player_address: &CStr,
    ) -> Result<PluginResult, String> {
        Ok(PluginResult::Continue)
    }
    fn client_command(&mut self, edict: *mut c_void, args: *const c_void) -> PluginResult {
        PluginResult::Continue
    }
    fn network_id_validated(&mut self, player_name: &CStr, network_id: &CStr) -> PluginResult {
        PluginResult::Continue
    }
    fn on_query_cvar_value_finished(
        &mut self,
        cookie: i32,
        edict: *mut c_void,
        status: QueryStatus,
        cvar_name: &CStr,
        cvar_value: &CStr,
    ) {
    }
    fn on_edict_allocated(&mut self, edict: *mut c_void) {}
    fn on_edict_freed(&mut self, edict: *const c_void) {}
}

// struct TestPlugin {
//     plugin_name: CString,
// }

// impl TestPlugin {
//     fn new() -> Self {
//         Self {
//             plugin_name: CString::new("TestPlugin").unwrap(),
//         }
//     }
// }

// impl ServerPlugin for TestPlugin {
//     fn get_plugin_description(&mut self) -> &CStr {
//         self.plugin_name.as_ref()
//     }
// }

// #[no_mangle]
// #[allow(unused_variables)]
// pub unsafe extern "C" fn CreateInterface(version: *const i8, result: *mut i32) -> *mut c_void {
//     mem::transmute(Box::new(BasePlugin::new(TestPlugin::new())))
// }
