use std::ffi::c_void;

use crate::{CreateInterfaceFn, PluginResult, QueryStatus};

#[allow(unused_variables)]
impl BasePlugin {
    fn new() -> Self {
        Self {
            server_callbacks_vtable: unsafe { SERVER_CALLBACKS_VTABLE.as_ptr() },
            string_tables: None
        }
    }
    #[inline(never)]
    unsafe extern "thiscall" fn load(
        &mut self,
        game_factory: CreateInterfaceFn,
        server_factory: CreateInterfaceFn,
    ) -> bool {
        self.string_tables = INetworkStringTableContainer::from_factory(game_factory).ok();
        true
    }
    #[inline(never)]
    unsafe extern "thiscall" fn unload(&mut self) {}
    #[inline(never)]
    unsafe extern "thiscall" fn pause(&mut self) {}
    #[inline(never)]
    unsafe extern "thiscall" fn unpause(&mut self) {}
    #[inline(never)]
    unsafe extern "thiscall" fn get_plugin_description(&mut self) -> *const i8 {
        b"BasePlugin\0".as_ptr() as _
    }
    #[inline(never)]
    unsafe extern "thiscall" fn level_init(&mut self, level_name: *const i8) {}
    #[inline(never)]
    unsafe extern "thiscall" fn server_activate(
        &mut self,
        edict_list: *mut c_void,
        edict_count: i32,
        max_clients: i32,
    ) {
        let string_tables = self.string_tables.unwrap();
        let lua_files = string_tables.find_table("client_lua_files").unwrap();
        let _ = lua_files.set_string_userdata(0, &[]);
    }
    #[inline(never)]
    unsafe extern "thiscall" fn game_frame(&mut self, simulating: bool) {
    }
    #[inline(never)]
    unsafe extern "thiscall" fn level_shutdown(&mut self) {}
    #[inline(never)]
    unsafe extern "thiscall" fn client_active(&mut self, edict: *mut c_void) {}
    #[inline(never)]
    unsafe extern "thiscall" fn client_disconnect(&mut self, edict: *mut c_void) {}
    #[inline(never)]
    unsafe extern "thiscall" fn client_put_in_server(
        &mut self,
        edict: *mut c_void,
        player_name: *const i8,
    ) {
    }
    #[inline(never)]
    unsafe extern "thiscall" fn set_command_client(&mut self, client_index: i32) {}
    #[inline(never)]
    unsafe extern "thiscall" fn client_settings_changed(&mut self, edict: *mut c_void) {}
    #[inline(never)]
    unsafe extern "thiscall" fn client_connect(
        &mut self,
        allow_connect: *mut bool,
        edict: *mut c_void,
        player_name: *const i8,
        player_address: *const i8,
        message_buffer: *mut i8,
        max_message_length: i32,
    ) -> PluginResult {
        dbg!(CStr::from_ptr(player_name), CStr::from_ptr(player_address));
        PluginResult::Continue
    }
    #[inline(never)]
    unsafe extern "thiscall" fn client_command(
        &mut self,
        edict: *mut c_void,
        args: *const c_void,
    ) -> PluginResult {
        PluginResult::Continue
    }
    #[inline(never)]
    unsafe extern "thiscall" fn network_id_validated(
        &mut self,
        player_name: *const i8,
        network_id: *const i8,
    ) -> PluginResult {
        PluginResult::Continue
    }
    #[inline(never)]
    unsafe extern "thiscall" fn on_query_cvar_value_finished(
        &mut self,
        cookie: i32,
        edict: *mut c_void,
        status: QueryStatus,
        cvar_name: *const i8,
        cvar_value: *const i8,
    ) {
    }
    #[inline(never)]
    unsafe extern "thiscall" fn on_edict_allocated(&mut self, edict: *mut c_void) {}
    #[inline(never)]
    unsafe extern "thiscall" fn on_edict_freed(&mut self, edict: *const c_void) {}
}